function stat = prepareEstimation(Z, nvoc)
% function stat = prepareEstimation(Z, nvoc)
%
% Input
%
% Z: input sequences, assumed to be generated by SWIRL.
%    N*nmax sparse matrix
%    The j-th row (j=1..N) is the j-th sequence Z^(j)=z_1^(j) ... z_{n(j)}^(j) which is n(j) elements long.
%    nmax >= max_{j=1..N} n(j).  it's OK to have zero padding in all rows.
%    The elements z_i^(j) are word type indices, encoded as positive integers 1, 2, 3, ...
%    Note: the variable-length sequences are padded with zero at the end.
%    Note: Z(j,i) corresponds to z_i^{(j)} in paper
%    Note: It's OK to have empty sequences
% nvoc: vocabulary size.
%
% Output
%
% stat: a structure containing constants required for computing SWIRL likelihood function.
%
% Reference
%
% Kwang-Sung Jun, Xiaojin Zhu, Burr Settles, and Timothy Rogers. Learning from Human-Generated 
% Lists. In The 30th International Conference on Machine Learning (ICML), 2013.
%
% Author
%
% Original: Jerry Zhu, July 2012
% Modified: Kwang-Sung Jun, Sep 2012

[N, ~]=size(Z); % N: number of sequences;
% nvoc=max(max(Z)); % vocabulary size

% construct the C matrices
% C{j}, j=1..N, is a nvoc*n(j) matrix for the j-th sequence.
% The (w,i)-th entry of the above matrix is the number of times w appears in z_1, ... z_{i-1}, the prefix of the j-th sequence
% Note: C{j}(w,i) corresponds to C^{(j)}_i(w) in paper

C=cell(N, 1);

n=zeros(N,1); % the length of each sequence

for j=1:N,
  n(j) = length(find(Z(j,:)));
  tmpC = sparse(nvoc, n(j));
  for i=2:n(j),
    tmpC(:,i) = tmpC(:,i-1);
    tmpC(Z(j,i-1),i) = tmpC(Z(j,i-1),i)+1;
  end
  C{j} = tmpC;
end


% compute the shorthand a and b
a = 0;
b = zeros(nvoc, 1);
for j=1:N,
  for i=1:n(j),
    a = a - C{j}(Z(j,i),i);
    b(Z(j,i)) = b(Z(j,i)) - 1;
  end
end

lambda = mean(n);

stat.a = a;
stat.b = b;
stat.C = C;
stat.N = N;
stat.lambda = lambda;
stat.n = n;
end